from . import PersianTextBase


class PersianCalendar(PersianTextBase):

    def translate_weekdays_to_number(self, include_common_styles: bool = True):
        """
        Translate weekdays to their number.
        based on python datetime, weekday numbers starts from 0 to 6.
        references:
            https://docs.python.org/3/library/datetime.html#datetime.date.weekday
            https://en.wikipedia.org/wiki/Names_of_the_days_of_the_week
        :return:
        """
        pattern_name = 'persian_weekdays_name_replace'
        words_map = {
            'شنبه': '0',
            'یکشنبه': '1',
            'دوشنبه': '2',
            'سه شنبه': '3',
            'چهارشنبه': '4',
            'پنجشنبه': '5',
            'جمعه': '6',
            'آدینه': '6'
        }
        if include_common_styles:
            pattern_name = 'persian_all_weekdays_name_replace'
            words_map.update(
                {
                    'یک شنبه': '1',
                    'دو شنبه': '2',
                    'دوشنبه': '2',
                    'چهار شنبه': '4',
                    'چارشنبه': '4',
                    'پنج شنبه': '5'
                }
            )

        self.replace_by_character_map(
            pattern_name,
            words_map
        )
        return self



import re


def multiple_replace(dic: dict, text: str, pattern: bytes):
    return re.sub(pattern, lambda m: dic[m.group().strip()], str(text))


def generate_multiple_replace_pattern(dictionary):
    pattern = "|".join(map(re.escape, dictionary.keys()))
    pattern = "(%s)" % pattern
    return pattern



import unittest
from neviseh import EnglishText


class EnglishTextTestCase(unittest.TestCase):

    def test_base(self):
        # Class representation
        self.assertEqual(repr(EnglishText('Hello')), 'EnglishText("Hello")')
        self.assertEqual(repr(EnglishText('Hello').translate_keymap()), 'EnglishText("Hello")')

    def test_keymap(self):
        self.assertEqual(
            str(EnglishText('لخخلمث').translate_keymap()),
            'google'
        )

    def test_numbers(self):
        self.assertEqual(
            str(EnglishText('0123456789۰۱۲۳۴۵۶۷۸۹').translate_persian_numbers()),
            '01234567890123456789'
        )


if __name__ == '__main__':  # pragma: no cover
    unittest.main()



import unittest
from neviseh import Text


class TextTestCase(unittest.TestCase):

    def test_base(self):
        # Class representation
        self.assertEqual(repr(Text('Hello')), 'Text("Hello")')
        self.assertEqual(repr(Text('Hello').remove_extra_spaces()), 'Text("Hello")')

    def test_normalization(self):

        # Remove extra spaces
        self.assertEqual(
            str(Text('Lorem ipsum  ').remove_extra_spaces()),
            'Lorem ipsum '
        )

        # Remove extra newlines
        self.assertEqual(
            str(Text('Lorem \n\n\n ipsum\n').remove_extra_newlines()),
            'Lorem \n ipsum\n'
        )

        # Replace three dots
        self.assertEqual(
            str(Text('Show more ...').replace_three_dots()),
            'Show more …'
        )

        # Apply all normalizations
        self.assertEqual(
            str(Text('Show more   ...').normalize()),
            'Show more …'
        )


if __name__ == '__main__':  # pragma: no cover
    unittest.main()



from . import EnglishTextBase


class EnglishNumbers(EnglishTextBase):

    def translate_persian_numbers(self):
        """
        Translate Persian numbers to Latin
        :return:
        """
        characters_map = {
            '۰': '0',
            '۱': '1',
            '۲': '2',
            '۳': '3',
            '۴': '4',
            '۵': '5',
            '۶': '6',
            '۷': '7',
            '۸': '8',
            '۹': '9',
            '.': '.',
            '٪': '%',
        }
        self.replace_by_character_map('persian_numbers_replace', characters_map)
        return self



import re
from typing import Union
from .utils import multiple_replace, generate_multiple_replace_pattern
from .pattern import PatternDict


class TextBase:
    language = None
    _initial_value = None
    iso_639_1_code = None

    def __init__(self, value: Union[str, int, float]):
        self.value = self._initial_value = str(value)
        self.patterns = PatternDict()

    def __str__(self):
        return self.value

    def __repr__(self):
        return 'Text("%s")' % self._initial_value

    def replace_by_character_map(self, pattern_name: str, characters_map: dict):
        """
        Replace value with dictionary of characters
        :param pattern_name: Pattern name will cache on memory
        :param characters_map: Characters map dictionary
        :return:
        """
        if pattern_name not in self.patterns:
            self.patterns[pattern_name] = generate_multiple_replace_pattern(characters_map)

        self.value = multiple_replace(
           characters_map, self.value, self.patterns[pattern_name]
        )
        return self


class TextNormalization(TextBase):

    def __init__(self, value):
        super().__init__(value)
        self.patterns.update({
            'extra_spaces': ' +',
            'extra_newlines': '\n\n+',
            'replace_three_dots': ' ?\.\.\.'
        })

    def replace_three_dots(self):
        """
        Replace three dots with unicode ellipsis
        :return:
        """
        self.value = re.sub(self.patterns['replace_three_dots'], ' …', self.value)
        return self

    def remove_extra_spaces(self):
        """
        Replace extra spaces
        :return:
        """
        self.value = re.sub(self.patterns['extra_spaces'], ' ', self.value)
        return self

    def remove_extra_newlines(self):
        """
        Replace extra new lines
        :return:
        """
        self.value = re.sub(self.patterns['extra_newlines'], '\n', self.value)
        return self

    def normalize(self):
        """
        Apply all normalizations
        :return:
        """
        self.remove_extra_newlines()
        self.remove_extra_spaces()
        self.replace_three_dots()
        return self


class Text(TextNormalization):
    pass



import unittest
from neviseh import PersianText


class PersianTextTestCase(unittest.TestCase):

    def test_base(self):
        # Class representation
        self.assertEqual(repr(PersianText('سلام')), 'PersianText("سلام")')
        self.assertEqual(repr(PersianText('سلام').translate_keymap()), 'PersianText("سلام")')

    def test_keymap(self):
        self.assertEqual(
            str(PersianText('sghl').translate_keymap()),
            'سلام'
        )

    def test_normalization(self):
        # Remove keshida
        self.assertEqual(
            str(PersianText('ســـــلام').remove_keshida()),
            'سلام'
        )

        # Remove diacritics
        self.assertEqual(
            str(PersianText('بُشقابِ مَن را بِگیر').remove_diacritics()),
            'بشقاب من را بگیر'
        )

        # Apply punctuation spacing
        self.assertEqual(
            str(PersianText('اصلاح ( پرانتزها ) در متن .').apply_punctuation_spacing()),
            'اصلاح (پرانتزها) در متن.'
        )
        self.assertEqual(
            str(PersianText('نسخه 0.5 در ساعت 22:00 تهران،1396').apply_punctuation_spacing()),
            'نسخه 0.5 در ساعت 22:00 تهران، 1396'
        )

        # Replace quotation with gyoomeh
        self.assertEqual(
            str(PersianText('نام او "جک" است').replace_quotation()),
            'نام او «جک» است'
        )

        # Replace decimal dots with Momayez
        self.assertEqual(
            str(PersianText('مبلغ برابر با 4.2 میلیارد است').replace_decimal_dots()),
            'مبلغ برابر با 4٫2 میلیارد است'
        )

        # Replace replace arabic letters
        self.assertEqual(
            str(PersianText('اصلاح كاف و ىاي عربي و سایر چیزهای مشابه').replace_similar_letters()),
            'اصلاح کاف و یای عربی و سایر چیزهای مشابه'
        )

        # Apply all
        self.assertEqual(
            str(PersianText('اصلاح كــاف و ىاي عربي و سایر چیزهای مشابه').normalize()),
            'اصلاح کاف و یای عربی و سایر چیزهای مشابه'
        )

    def test_numbers(self):

        # Translate latin numbers
        self.assertEqual(
            str(PersianText('0123456789۰۱۲۳۴۵۶۷۸۹').translate_latin_numbers()),
            '۰۱۲۳۴۵۶۷۸۹۰۱۲۳۴۵۶۷۸۹'
        )

        # Translate Arabic numbers
        self.assertEqual(
            str(PersianText('۰۱۲۳٤۵٦۷۸۹۰۱۲۳۴۵۶۷۸۹').translate_arabic_numbers()),
            '۰۱۲۳۴۵۶۷۸۹۰۱۲۳۴۵۶۷۸۹'
        )

        # Translate calendar weekdays
        self.assertEqual(
            str(PersianText('دوشنبه').translate_weekdays_to_number()),
            '2'
        )


if __name__ == '__main__':  # pragma: no cover
    unittest.main()



from . import EnglishTextBase


class EnglishKeymap(EnglishTextBase):

    def translate_keymap(self):
        """
        Translate Persian keyboard layout to English
        Note: Assumes that characters written with standard
              persian keyboard layout (ISIRI 2901).
              http://www.isiri.gov.ir/portal/files/std/9147.pdf
        :return: string
        """
        keymap_dict = {
            'خ': 'o',
            '؟': '?',
            'ک': ';',
            'ض': 'q',
            'ذ': 'b',
            'ز': 'c',
            'ح': 'p',
            'و': ',',
            'ف': 't',
            'چ': ']',
            'ج': '[',
            'پ': 'm',
            'ن': 'k',
            'ص': 'w',
            'س': 's',
            'ث': 'e',
            'ش': 'a',
            'ط': 'x',
            'گ': '\'',
            'ع': 'u',
            'غ': 'y',
            'ق': 'r',
            'م': 'l',
            'ی': 'd',
            'ظ': 'z',
            'د': 'n',
            'ه': 'i',
            'ل': 'g',
            'ر': 'v',
            'ت': 'j',
            'ا': 'h',
            'ب': 'f',
        }
        self.replace_by_character_map('english_keymap_replace', keymap_dict)
        return self



import re


class PatternDict:
    """
    In-memory dictionary of compiled patterns
    """
    __slots__ = ('patterns', )

    def compile_all(self):
        for pattern_name, pattern_value in self.patterns.items():
            if isinstance(pattern_value, str):
                self.patterns[pattern_name] = re.compile(pattern_value)

    def __init__(self, patterns: dict = None):
        self.patterns = patterns or {}
        self.compile_all()

    def __setitem__(self, key, value):
        if isinstance(value, str):
            value = re.compile(value)
        return self.patterns.__setitem__(key, value)

    def __delitem__(self, key):  # pragma: nocover
        return self.patterns.__delitem__(key)

    def __contains__(self, item):
        return self.patterns.__contains__(item)

    def __getitem__(self, item):
        return self.patterns.__getitem__(item)

    def __len__(self):  # pragma: nocover
        return self.patterns.__len__()

    def __iter__(self):  # pragma: nocover
        return self.patterns.__iter__()

    def update(self, dic: dict):
        self.patterns.update(dic)
        self.compile_all()



import re
from os.path import join, dirname
from setuptools import setup, find_packages
readme = open('README.rst').read()

# reading package version (without reloading it)
with open(join(dirname(__file__), 'neviseh', '__init__.py')) as v_file:
    package_version = re.compile(r".*__version__ = '(.*?)'", re.S).match(v_file.read()).group(1)

setup(
    name='neviseh',
    description='Simple text processing tools in persian',
    long_description=readme,
    version=package_version,
    author='Mahdi Ghane.g',
    license='MIT',
    keywords='text-processing persian iran localization',
    url='https://github.com/meyt/neviseh',
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Unix',
        'Natural Language :: Persian',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries',
        'Topic :: Text Processing'
    ],
    packages=find_packages()
)



import re
from . import PersianTextBase


class PersianTextNormalization(PersianTextBase):

    def __init__(self, value):
        super().__init__(value)
        punctuation_after, punctuation_before = (
            '\.:!،؛؟»\]\)\}',
            '«\[\(\{'
        )

        self.patterns.update({
            'remove_keshida': r'[ـ\r]',
            'remove_diacritics': r'[\u064B\u064C\u064D\u064E\u064F\u0650\u0651\u0652]',
            'replace_quotation': r'"([^\n"]+)"',
            'replace_decimal_dots': '([\d+])\.([\d+])',

            'remove_space_before_and_after_quotation': r'" ([^\n"]+) "',
            'remove_space_before': ' ([' + punctuation_after + '])',
            'remove_space_after': '([' + punctuation_before + ']) ',
            'put_space_after_dot': '([' + punctuation_after[:3] + '])([^ \d' + punctuation_after + '])',
            'put_space_after': '([' + punctuation_after[3:] + '])([^ ' + punctuation_after + '])',
            'put_space_before': '([^ ' + punctuation_before + '])([' + punctuation_before + '])',
        })

    def remove_keshida(self):
        """
        Remove Keshida/ARABIC_TATWEEL
        :return:
        """
        self.value = re.sub(self.patterns['remove_keshida'], '', self.value)
        return self

    def remove_diacritics(self):
        """
        Remove Arabic/Persian diacritics.
        includes: FATHATAN, DAMMATAN, KASRATAN, FATHA,
                  DAMMA, KASRA, SHADDA, SUKUN
        :return:
        """
        self.value = re.sub(self.patterns['remove_diacritics'], '', self.value)
        return self

    def apply_punctuation_spacing(self):
        """
        Apply punctuation spacing
        :return:
        """
        self.value = re.sub(self.patterns['remove_space_before_and_after_quotation'], r'"\1"', self.value)
        self.value = re.sub(self.patterns['remove_space_before'], r'\1', self.value)
        self.value = re.sub(self.patterns['remove_space_after'], r'\1', self.value)
        self.value = re.sub(self.patterns['put_space_after_dot'], r'\1 \2', self.value)
        self.value = re.sub(self.patterns['put_space_after'], r'\1 \2', self.value)
        self.value = re.sub(self.patterns['put_space_before'], r'\1 \2', self.value)
        return self

    def replace_quotation(self):
        """
        Replace quotation with gyoomeh
        :return:
        """
        self.value = re.sub(self.patterns['replace_quotation'], r'«\1»', self.value)
        return self

    def replace_decimal_dots(self):
        """
        Replace decimal Dot with Momayez
        :return:
        """
        self.value = re.sub(self.patterns['replace_decimal_dots'], r'\1٫\2', self.value)
        return self

    def replace_similar_letters(self):
        """
        Replace similar letters in persian
        :return:
        """
        characters_map = {
            'ك': 'ک',
            'دِ': 'د',
            'بِ': 'ب',
            'زِ': 'ز',
            'ذِ': 'ذ',
            'شِ': 'ش',
            'سِ': 'س',
            'ى': 'ی',
            'ي': 'ی'
        }
        self.replace_by_character_map('arabic_persian_letters_replace', characters_map)
        return self

    def normalize(self):
        """
        Apply all normalizations
        :return:
        """
        super().normalize()
        return (
            self.remove_keshida()
                .replace_similar_letters()
                .replace_quotation()
                .remove_diacritics()
                .apply_punctuation_spacing()
        )



from .base import PersianTextBase
from .calendar import PersianCalendar
from .keymap import PersianKeymap
from .numbers import PersianNumbers
from .text_normalization import PersianTextNormalization


class PersianText(
    PersianCalendar,
    PersianKeymap,
    PersianNumbers,
    PersianTextNormalization,
):
    pass



from . import PersianTextBase


class PersianKeymap(PersianTextBase):

    def translate_keymap(self):
        """
        Translate english keyboard layout to persian
        Note: Assumes that characters written with standard
        persian keyboard, not windows arabic layout.
        :return: string
        """
        keymap_dict = {
            'q': 'ض',
            'w': 'ص',
            'e': 'ث',
            'r': 'ق',
            't': 'ف',
            'y': 'غ',
            'u': 'ع',
            'i': 'ه',
            'o': 'خ',
            'p': 'ح',
            '[': 'ج',
            ']': 'چ',
            'a': 'ش',
            's': 'س',
            'd': 'ی',
            'f': 'ب',
            'g': 'ل',
            'h': 'ا',
            'j': 'ت',
            'k': 'ن',
            'l': 'م',
            ';': 'ک',
            "'": 'گ',
            'z': 'ظ',
            'x': 'ط',
            'c': 'ز',
            'v': 'ر',
            'b': 'ذ',
            'n': 'د',
            'm': 'پ',
            ',': 'و',
            '?': '؟',
        }
        self.replace_by_character_map('persian_keymap_replace', keymap_dict)
        return self



from . import PersianTextBase


class PersianNumbers(PersianTextBase):

    def translate_arabic_numbers(self):
        """
        Translate Arabic numbers to Persian
        :return:
        """
        characters_map = {
            '١': '۱',
            '٢': '۲',
            '٣': '۳',
            '٤': '۴',
            '٥': '۵',
            '٦': '۶',
            '٧': '۷',
            '٨': '۸',
            '٩': '۹',
            '٠': '۰',
        }
        self.replace_by_character_map('arabic_numbers_replace', characters_map)
        return self

    def translate_latin_numbers(self):
        """
        Translate Latin numbers to Persian
        :return:
        """
        characters_map = {
            '0': '۰',
            '1': '۱',
            '2': '۲',
            '3': '۳',
            '4': '۴',
            '5': '۵',
            '6': '۶',
            '7': '۷',
            '8': '۸',
            '9': '۹',
            '.': '.',
            '%': '٪'
        }
        self.replace_by_character_map('latin_numbers_replace', characters_map)
        return self
